import React, { Fragment } from 'react';
import { Divider, message, Form, Popconfirm } from 'antd';
import moment from 'moment';
import router from 'umi/router';
import ExportInfo from '../Export/index';
import { text, number, date, getFormArrayConfig } from '../config/index';
import ButtonDiy from '@/baseComponent/ButtonDiy';
import Shell from '../Shell';
import Edit from '../Edit/index';
import ImportUtil from '../ImportUtil/ImportUtil';
import SearchDom from '@/highOrderComponent/SearchDom';
import StandardTable from '@/components/StandardTable';
import { getHead, fetchData, deleteItem, add } from './services';
import AddOrEditModal from './AddOrEditModal';

const getValue = obj =>
  Object.keys(obj)
    .map(key => obj[key])
    .join(',');

class ListWithAddEditTemplate extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      modalVisible: false,
      formData: null,
      headerList: [],
      searchConfig: [],
      formValues: {},
      isAdd: false,
      selectedRows: [],
      primaryKey: null,
      showEdit: false,
      pageData: {
        list: [],
        pagination: {},
      },
    };
    this.columns = [];
  }

  // 渲染值
  componentDidMount() {
    const { objId } = this.props;
    getHead({ dataObjId: objId }, datas => {
      if (!datas) return;
      this.setState({ headerList: datas });
      let i = 0;
      for (let t in datas) {
        if (i < 10) {
          let column = {};
          column.title = datas[t].title;
          column.dataIndex = datas[t].name;
          if (date.includes(datas[t].type)) {
            column.render = val => val && moment(val).format('YYYY-MM-DD HH:mm:ss');
          }
          this.columns.push(column);
          i++;
        }
        if (datas[t].isPrimaryKey) {
          this.setState({
            primaryKey: datas[t].name,
          });
        }
      }
      let searchConfig = [];
      for (let item of datas) {
        if (item.isShowQuery) {
          let sear = getFormArrayConfig([item]);
          if (!Array.isArray(sear)) {
            return false;
          }
          delete sear[0].required;
          delete sear[0].placeHolder;
          searchConfig.push(sear[0]);
        }
      }
      this.setState({
        searchConfig,
      });
      let opt = {
        title: '操作',
        render: (text, record) => {
          return (
            <Fragment>
              <div>
                <a onClick={this.modify.bind(this, record)}>编辑</a>
                <Divider type="vertical" />
                <Popconfirm
                  title="确定删除该数据?"
                  onConfirm={this.delete.bind(this, record)}
                  okText="确定"
                  cancelText="取消"
                >
                  <a>删除</a>
                </Popconfirm>
              </div>
            </Fragment>
          );
        },
      };
      this.columns.push(opt);
      this.getPage();
    });
  }

  // 新增按钮事件
  add = () => {
    const { headerList } = this.state;
    const showEdit = headerList.length >= 10;
    this.setState({
      modalVisible: !showEdit,
      formData: {},
      isAdd: true,
      showEdit,
    });
  };

  // 修改按钮事件
  modify = record => {
    const recordTemp = { ...record, update_time: Date.now() };
    const { headerList } = this.state;
    const showEdit = headerList.length >= 10;
    // 把record里明文key改成转换后的key，用于在模板表单中回显
    const headerMap = headerList.reduce((acc, header) => ({ ...acc, [header.name]: header }), {});
    const formData = Object.keys(recordTemp).reduce(
      (acc, key) => ({ ...acc, [headerMap[key].base52]: recordTemp[key] }),
      {}
    );
    this.setState({
      formData,
      modalVisible: !showEdit,
      isAdd: false,
      showEdit,
    });
  };

  delete = record => {
    const { objId } = this.props;
    // 找出主键
    const { primaryKey } = this.state;
    const Keys = {};
    Keys[primaryKey] = record[primaryKey];
    deleteItem({ keys: JSON.stringify(Keys), objId }, () => {
      this.getPage();
    });
  };

  changeFormData = (value, key) => {
    this.setState(({ formData }) => ({ formData: { ...formData, [key]: value } }));
  };

  getPage = () => {
    const { objId, filters = {} } = this.props;
    const { formValues, pageData: pagination } = this.state;
    const params = {
      query: JSON.stringify({ ...formValues, ...filters }),
      pageNo: pagination.current ? pagination.current : 1,
      pageSize: pagination.pageSize ? pagination.pageSize : 10,
      dataObjId: objId,
    };

    fetchData(params, res => {
      if (!res.rows) {
        console.log('分页接口rows返回null报错');
      }
      this.setState({
        pageData: {
          list: res.rows,
          pagination: {
            current: res.pageNo,
            pageSize: res.pageSize,
            total: Number(res.total),
          },
        },
      });
    });
  };

  handleStandardTableChange = (pagination, filtersArg, sorter) => {
    const filters = Object.keys(filtersArg).reduce((obj, key) => {
      const newObj = { ...obj };
      newObj[key] = getValue(filtersArg[key]);
      return newObj;
    }, {});

    const params = {
      pageNo: pagination.current,
      pageSize: pagination.pageSize,
      ...filters,
    };
    if (sorter.field) {
      params.sorter = `${sorter.field}_${sorter.order}`;
    }
    this.getPage(params);
  };

  handleAdd = (fields, isAdd) => {
    const {
      objId,
      form: { validateFields },
    } = this.props;
    validateFields((err, values) => {
      if (err) return;
      add({ params: values, objId, isAdd, isBase: true }, () => {
        message.success('操作成功');
        this.setState({
          modalVisible: false,
        });
        this.getPage();
      });
    });
  };

  handleModalVisible = () => {
    this.setState({
      modalVisible: this.state.modalVisible ? false : true,
      formData: {},
    });
  };

  handleSelectRows = rows => {
    this.setState({
      selectedRows: rows,
    });
  };

  batchDelete = e => {
    const { selectedRows, formItem, objId } = this.state;

    if (!selectedRows) return;
    const Keys = formItem.filter(i => i.isPrimaryKey).reduce((acc, i) => {
      return { ...acc, [i.name]: selectedRows.map(row => row[i.name]).join(',') };
    });

    deleteItem({ keys: JSON.stringify(Keys), objId }, () => {
      this.setState({
        selectedRows: [],
      });
      this.getPage();
    });
  };

  goBack = () => {
    router.goBack();
  };

  formStateChange = (value, key) => {
    this.setState(({ formValues }) => ({ formValues: { ...formValues, [key]: value } }));
  };

  resetFormValues = () => {
    this.setState({
      formValues: {},
    });
  };

  render() {
    const {
      modalVisible,
      selectedRows,
      isAdd,
      objId,
      pageData,
      showEdit,
      formData,
      primaryKey,
      formValues,
      searchConfig,
    } = this.state;
    const { hasExport, form, hasImport, templateCode } = this.props;

    const parentMethods = {
      handleAdd: this.handleAdd,
      handleModalVisible: this.handleModalVisible,
      content: formData,
      form,
      isAdd,
      templateCode,
    };
    return (
      <>
        {showEdit ? (
          <Edit
            id={objId}
            recordId={formData[primaryKey]}
            recordKey={primaryKey}
            returnThis={() => {
              this.setState({
                showEdit: false,
              });
              this.getPage();
            }}
          />
        ) : (
          <Fragment>
            {searchConfig.length > 0 ? (
              <SearchDom
                formStateChange={this.formStateChange}
                formValues={formValues}
                getPage={this.getPage}
                resetFormValues={this.resetFormValues}
                config={{ condition: searchConfig, fromTab: true }}
              />
            ) : null}
            <Shell>
              <div style={{ paddingLeft: '10px', marginTop: '20px' }}>
                <ButtonDiy icon="plus" className="primaryBlue" handleClick={this.add} name="新建" />
                {hasImport && <ImportUtil objId={objId} />}
                {hasExport && <ExportInfo objId={objId} />}
                <ButtonDiy handleClick={this.batchDelete} name="批量删除" />
              </div>
              <StandardTable
                rowKey="id"
                data={pageData}
                columns={this.columns}
                selectedRows={selectedRows}
                onSelectRow={this.handleSelectRows}
                onChange={this.handleStandardTableChange}
              />
            </Shell>
          </Fragment>
        )}
        <AddOrEditModal {...parentMethods} modalVisible={modalVisible} />
      </>
    );
  }
}

export default Form.create()(ListWithAddEditTemplate);
